@@ -3,31 +3,48 @@ import pandas as pd
 import numpy as np
 from napoleontoolbox.utility import metrics
 
-def indice_rebalancing_kpi(weight_data= None, starting_date=None, ending_date=None):
-    indice_track = weight_data['indice']
-    indice_track=indice_track.dropna()
-    if starting_date is None:
-        starting_date = min(indice_track.index)
+def past_volatility(x):
+    return np.sqrt(252)*np.std(x)
+
+def vol_target_indice_rebalancing(weight_data=None, vol_target = None, vol_target_inception_date = None, initial_value = 100.):
+    assert vol_target is not None
+    weight_data['indice_returns'] = weight_data['indice'].pct_change()
+    weight_data['rolling_vol_indice'] = weight_data['indice_returns'].rolling(252).apply(past_volatility)
+    is_past_volatility_not_available_at_start =  np.isnan(weight_data.loc[vol_target_inception_date,'rolling_vol_indice'])
+    assert not is_past_volatility_not_available_at_start
+    print(f'date range before filtering {min(weight_data.index)} {max(weight_data.index)}')
+    weight_data = weight_data.loc[weight_data.index >= vol_target_inception_date, :]
+    print(f'date range after filtering {min(weight_data.index)} {max(weight_data.index)}')
+    assert vol_target_inception_date in weight_data.index
+
+    volatility_factor = None
+    target_vol_values_list = []
+    for date_index, row in weight_data.iterrows():
+        if date_index == vol_target_inception_date:
+            assert row['is_rebalancing']
+            volatility_factor = vol_target / weight_data.loc[vol_target_inception_date,'rolling_vol_indice']
+            previous_value = initial_value
+            target_vol_values_list.append({'Date': vol_target_inception_date, 'indice': initial_value, 'volatility_factor':np.nan})
+            continue
+
+        current_return = weight_data.loc[date_index, 'indice_returns']
+        value = previous_value * (1 + current_return * volatility_factor)
+        current_values = {'Date' : date_index, 'indice': value, 'volatility_factor':volatility_factor}
+        target_vol_values_list.append(current_values)
+        # we readjust the volatility factor    
+        if row['is_rebalancing']:
+            volatility_factor = vol_target / weight_data.loc[date_index,'rolling_vol_indice']
+        previous_value = value
+
+    target_vol_values_df = pd.DataFrame(target_vol_values_list)
+    target_vol_values_df.index = pd.to_datetime(target_vol_values_df['Date'])
+    target_vol_values_df.drop('Date', axis=1, inplace=True)
+    return target_vol_values_df
+
 
-    if ending_date is None:
-        ending_date = max(indice_track.index)
 
-    starting_value = indice_track.loc[starting_date]
-    ending_value = indice_track.loc[ending_date]
 
-    days_lapse = (ending_date - starting_date).days
 
-    annualized_return = (ending_value/starting_value)**(365/(days_lapse))-1
-    vol = np.sqrt(252)*np.std(indice_track.loc[starting_date:ending_date].pct_change().fillna(0.).values)
-    annualized_return = annualized_return * 100
-    vol = vol*100
-    dd = max(metrics.drawdown(indice_track.loc[starting_date:ending_date]))*100
-    result_df = pd.DataFrame({
-        'volatility':[vol],
-        'drawdown':[dd],
-        'annualized_return':[annualized_return]
-    })
-    return result_df
 
 
 def indice_rebalancing(price_data= None, inception_date=None, target_weights=None, initial_value= 100., rebalancing_method = None):
@@ -39,10 +56,8 @@ def indice_rebalancing(price_data= None, inception_date=None, target_weights=Non
     price_data = price_data.loc[price_data.index >= inception_date, :]
     print(f'date range after filtering {min(price_data.index)} {max(price_data.index)}')
     assert inception_date in price_data.index
-
     date_utility.add_rebalancing_datepart(price_data, 'Date', rebalancing_method = rebalancing_method)
     assert price_data.shape[0] > 0
-
     initial_weights = target_weights.copy()
     initial_weights.update({
         'Date' : inception_date,
@@ -81,10 +96,35 @@ def indice_rebalancing(price_data= None, inception_date=None, target_weights=Non
 
     weights_df = pd.DataFrame(weights_list)
     weights_df.index = pd.to_datetime(weights_df['Date'])
-    weights_df.drop('Date', axis=1, inplace=True)
+    date_utility.add_rebalancing_datepart(weights_df, 'Date', rebalancing_method = rebalancing_method)
 
     for me_constituent in constituents:
         weights_df[me_constituent] = weights_df[me_constituent] / weights_df['indice']
 
     weights_df['indice'] = weights_df['indice']*initial_value
     return weights_df
+
+
+def indice_rebalancing_kpi(weight_data= None, starting_date=None, ending_date=None):
+    indice_track = weight_data['indice']
+    indice_track=indice_track.dropna()
+    if starting_date is None:
+        starting_date = min(indice_track.index)
+    if ending_date is None:
+        ending_date = max(indice_track.index)
+    starting_value = indice_track.loc[starting_date]
+    ending_value = indice_track.loc[ending_date]
+
+    days_lapse = (ending_date - starting_date).days
+
+    annualized_return = (ending_value/starting_value)**(365/(days_lapse))-1
+    vol = np.sqrt(252)*np.std(indice_track.loc[starting_date:ending_date].pct_change().fillna(0.).values)
+    annualized_return = annualized_return * 100
+    vol = vol*100
+    dd = max(metrics.drawdown(indice_track.loc[starting_date:ending_date]))*100
+    result_df = pd.DataFrame({
+        'volatility':[vol],
+        'drawdown':[dd],
+        'annualized_return':[annualized_return]
+    })
+    return result_df
\ No newline at end of file
